from enum import IntEnum


class PasswordType(IntEnum):
    PLAINTEXT = 0
    MD5_UNSALTED = 1
    SHA1_UNSALTED = 2
    SHA256_UNSALTED = 3
    TripleDES = 4
    IPBoard_MyBB = 5
    VBulletinPre3_8_5 = 6
    VBulletinPost3_8_5 = 7
    BCrypt = 8
    CRC32 = 9
    PHPBB3 = 10
    CustomAlgorithm1 = 11
    SCrypt = 12
    CustomAlgorithm2 = 13
    SHA512 = 14
    CustomAlgorithm3 = 15
    MD5Crypt = 16
    CustomAlgorithm4 = 17
    CustomAlgorithm5 = 18
    osCommerce_AEF = 19
    DESCrypt = 20
    MySQLPre4_1 = 21
    MySQLPost4_1 = 22
    PeopleSoft = 23
    PunBB = 24
    CustomAlgorithm6 = 25
    PartialMD5_20 = 26
    AVE_DataLife_Diferior = 27
    DjangoMD5 = 28
    DjangoSHA1 = 29
    PartialMD5_29 = 30
    PliggCMS = 31
    RunCMS_SMF1_1 = 32
    NTLM = 33
    SHA1Dash = 34
    SHA384 = 35
    CustomAlgorithm7 = 36
    CustomAlgorithm8 = 37
    CustomAlgorithm9 = 38
    SHA512Crypt = 39
    CustomAlgorithm10 = 40
    HMACSHA1_SaltAsKey = 41
    AuthMeSHA256 = 42
    SHA256Crypt = 43
    Unknown = 97
    UnusablePassword = 98
